# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.accounts import member_list_params, member_create_params, member_update_params
from ...types.shared.member import Member
from ...types.shared_params.role import Role
from ...types.accounts.member_delete_response import MemberDeleteResponse

__all__ = ["MembersResource", "AsyncMembersResource"]


class MembersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MembersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return MembersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MembersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return MembersResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        account_id: str,
        email: str,
        roles: SequenceNotStr[str],
        status: Literal["accepted", "pending"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        """
        Add a user to the list of members for this account.

        Args:
          account_id: Account identifier tag.

          email: The contact email address of the user.

          roles: Array of roles associated with this member.

          status: Status of the member invitation. If not provided during creation, defaults to
              'pending'. Changing from 'accepted' back to 'pending' will trigger a replacement
              of the member resource in Terraform.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        account_id: str,
        email: str,
        policies: Iterable[member_create_params.IAMCreateMemberWithPoliciesPolicy],
        status: Literal["accepted", "pending"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        """
        Add a user to the list of members for this account.

        Args:
          account_id: Account identifier tag.

          email: The contact email address of the user.

          policies: Array of policies associated with this member.

          status: Status of the member invitation. If not provided during creation, defaults to
              'pending'. Changing from 'accepted' back to 'pending' will trigger a replacement
              of the member resource in Terraform.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "email", "roles"], ["account_id", "email", "policies"])
    def create(
        self,
        *,
        account_id: str,
        email: str,
        roles: SequenceNotStr[str] | Omit = omit,
        status: Literal["accepted", "pending"] | Omit = omit,
        policies: Iterable[member_create_params.IAMCreateMemberWithPoliciesPolicy] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/members", account_id=account_id),
            body=maybe_transform(
                {
                    "email": email,
                    "roles": roles,
                    "status": status,
                    "policies": policies,
                },
                member_create_params.MemberCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Member]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Member]], ResultWrapper[Member]),
        )

    @overload
    def update(
        self,
        member_id: str,
        *,
        account_id: str,
        roles: Iterable[Role] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        """
        Modify an account member.

        Args:
          account_id: Account identifier tag.

          member_id: Membership identifier tag.

          roles: Roles assigned to this member.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        member_id: str,
        *,
        account_id: str,
        policies: Iterable[member_update_params.IAMUpdateMemberWithPoliciesPolicy],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        """
        Modify an account member.

        Args:
          account_id: Account identifier tag.

          member_id: Membership identifier tag.

          policies: Array of policies associated with this member.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id"], ["account_id", "policies"])
    def update(
        self,
        member_id: str,
        *,
        account_id: str,
        roles: Iterable[Role] | Omit = omit,
        policies: Iterable[member_update_params.IAMUpdateMemberWithPoliciesPolicy] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not member_id:
            raise ValueError(f"Expected a non-empty value for `member_id` but received {member_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/members/{member_id}", account_id=account_id, member_id=member_id),
            body=maybe_transform(
                {
                    "roles": roles,
                    "policies": policies,
                },
                member_update_params.MemberUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Member]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Member]], ResultWrapper[Member]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        order: Literal["user.first_name", "user.last_name", "user.email", "status"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        status: Literal["accepted", "pending", "rejected"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[Member]:
        """
        List all members of an account.

        Args:
          account_id: Account identifier tag.

          direction: Direction to order results.

          order: Field to order results by.

          page: Page number of paginated results.

          per_page: Maximum number of results per page.

          status: A member's status in the account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/members", account_id=account_id),
            page=SyncV4PagePaginationArray[Member],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    member_list_params.MemberListParams,
                ),
            ),
            model=Member,
        )

    def delete(
        self,
        member_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MemberDeleteResponse]:
        """
        Remove a member from an account.

        Args:
          account_id: Account identifier tag.

          member_id: Membership identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not member_id:
            raise ValueError(f"Expected a non-empty value for `member_id` but received {member_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/members/{member_id}", account_id=account_id, member_id=member_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MemberDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MemberDeleteResponse]], ResultWrapper[MemberDeleteResponse]),
        )

    def get(
        self,
        member_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        """
        Get information about a specific member of an account.

        Args:
          account_id: Account identifier tag.

          member_id: Membership identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not member_id:
            raise ValueError(f"Expected a non-empty value for `member_id` but received {member_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/members/{member_id}", account_id=account_id, member_id=member_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Member]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Member]], ResultWrapper[Member]),
        )


class AsyncMembersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMembersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMembersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMembersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncMembersResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        account_id: str,
        email: str,
        roles: SequenceNotStr[str],
        status: Literal["accepted", "pending"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        """
        Add a user to the list of members for this account.

        Args:
          account_id: Account identifier tag.

          email: The contact email address of the user.

          roles: Array of roles associated with this member.

          status: Status of the member invitation. If not provided during creation, defaults to
              'pending'. Changing from 'accepted' back to 'pending' will trigger a replacement
              of the member resource in Terraform.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        account_id: str,
        email: str,
        policies: Iterable[member_create_params.IAMCreateMemberWithPoliciesPolicy],
        status: Literal["accepted", "pending"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        """
        Add a user to the list of members for this account.

        Args:
          account_id: Account identifier tag.

          email: The contact email address of the user.

          policies: Array of policies associated with this member.

          status: Status of the member invitation. If not provided during creation, defaults to
              'pending'. Changing from 'accepted' back to 'pending' will trigger a replacement
              of the member resource in Terraform.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "email", "roles"], ["account_id", "email", "policies"])
    async def create(
        self,
        *,
        account_id: str,
        email: str,
        roles: SequenceNotStr[str] | Omit = omit,
        status: Literal["accepted", "pending"] | Omit = omit,
        policies: Iterable[member_create_params.IAMCreateMemberWithPoliciesPolicy] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/members", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "email": email,
                    "roles": roles,
                    "status": status,
                    "policies": policies,
                },
                member_create_params.MemberCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Member]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Member]], ResultWrapper[Member]),
        )

    @overload
    async def update(
        self,
        member_id: str,
        *,
        account_id: str,
        roles: Iterable[Role] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        """
        Modify an account member.

        Args:
          account_id: Account identifier tag.

          member_id: Membership identifier tag.

          roles: Roles assigned to this member.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        member_id: str,
        *,
        account_id: str,
        policies: Iterable[member_update_params.IAMUpdateMemberWithPoliciesPolicy],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        """
        Modify an account member.

        Args:
          account_id: Account identifier tag.

          member_id: Membership identifier tag.

          policies: Array of policies associated with this member.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id"], ["account_id", "policies"])
    async def update(
        self,
        member_id: str,
        *,
        account_id: str,
        roles: Iterable[Role] | Omit = omit,
        policies: Iterable[member_update_params.IAMUpdateMemberWithPoliciesPolicy] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not member_id:
            raise ValueError(f"Expected a non-empty value for `member_id` but received {member_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/members/{member_id}", account_id=account_id, member_id=member_id),
            body=await async_maybe_transform(
                {
                    "roles": roles,
                    "policies": policies,
                },
                member_update_params.MemberUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Member]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Member]], ResultWrapper[Member]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        order: Literal["user.first_name", "user.last_name", "user.email", "status"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        status: Literal["accepted", "pending", "rejected"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Member, AsyncV4PagePaginationArray[Member]]:
        """
        List all members of an account.

        Args:
          account_id: Account identifier tag.

          direction: Direction to order results.

          order: Field to order results by.

          page: Page number of paginated results.

          per_page: Maximum number of results per page.

          status: A member's status in the account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/members", account_id=account_id),
            page=AsyncV4PagePaginationArray[Member],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    member_list_params.MemberListParams,
                ),
            ),
            model=Member,
        )

    async def delete(
        self,
        member_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MemberDeleteResponse]:
        """
        Remove a member from an account.

        Args:
          account_id: Account identifier tag.

          member_id: Membership identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not member_id:
            raise ValueError(f"Expected a non-empty value for `member_id` but received {member_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/members/{member_id}", account_id=account_id, member_id=member_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MemberDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MemberDeleteResponse]], ResultWrapper[MemberDeleteResponse]),
        )

    async def get(
        self,
        member_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Member]:
        """
        Get information about a specific member of an account.

        Args:
          account_id: Account identifier tag.

          member_id: Membership identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not member_id:
            raise ValueError(f"Expected a non-empty value for `member_id` but received {member_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/members/{member_id}", account_id=account_id, member_id=member_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Member]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Member]], ResultWrapper[Member]),
        )


class MembersResourceWithRawResponse:
    def __init__(self, members: MembersResource) -> None:
        self._members = members

        self.create = to_raw_response_wrapper(
            members.create,
        )
        self.update = to_raw_response_wrapper(
            members.update,
        )
        self.list = to_raw_response_wrapper(
            members.list,
        )
        self.delete = to_raw_response_wrapper(
            members.delete,
        )
        self.get = to_raw_response_wrapper(
            members.get,
        )


class AsyncMembersResourceWithRawResponse:
    def __init__(self, members: AsyncMembersResource) -> None:
        self._members = members

        self.create = async_to_raw_response_wrapper(
            members.create,
        )
        self.update = async_to_raw_response_wrapper(
            members.update,
        )
        self.list = async_to_raw_response_wrapper(
            members.list,
        )
        self.delete = async_to_raw_response_wrapper(
            members.delete,
        )
        self.get = async_to_raw_response_wrapper(
            members.get,
        )


class MembersResourceWithStreamingResponse:
    def __init__(self, members: MembersResource) -> None:
        self._members = members

        self.create = to_streamed_response_wrapper(
            members.create,
        )
        self.update = to_streamed_response_wrapper(
            members.update,
        )
        self.list = to_streamed_response_wrapper(
            members.list,
        )
        self.delete = to_streamed_response_wrapper(
            members.delete,
        )
        self.get = to_streamed_response_wrapper(
            members.get,
        )


class AsyncMembersResourceWithStreamingResponse:
    def __init__(self, members: AsyncMembersResource) -> None:
        self._members = members

        self.create = async_to_streamed_response_wrapper(
            members.create,
        )
        self.update = async_to_streamed_response_wrapper(
            members.update,
        )
        self.list = async_to_streamed_response_wrapper(
            members.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            members.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            members.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["ManagedTransformEditParams", "ManagedRequestHeader", "ManagedResponseHeader"]


class ManagedTransformEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """The unique ID of the zone."""

    managed_request_headers: Required[Iterable[ManagedRequestHeader]]
    """The list of Managed Request Transforms."""

    managed_response_headers: Required[Iterable[ManagedResponseHeader]]
    """The list of Managed Response Transforms."""


class ManagedRequestHeader(TypedDict, total=False):
    """A Managed Transform object."""

    id: Required[str]
    """The human-readable identifier of the Managed Transform."""

    enabled: Required[bool]
    """Whether the Managed Transform is enabled."""


class ManagedResponseHeader(TypedDict, total=False):
    """A Managed Transform object."""

    id: Required[str]
    """The human-readable identifier of the Managed Transform."""

    enabled: Required[bool]
    """Whether the Managed Transform is enabled."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["DiscoveryOperation", "Features", "FeaturesTrafficStats"]


class FeaturesTrafficStats(BaseModel):
    last_updated: datetime

    period_seconds: int
    """The period in seconds these statistics were computed over"""

    requests: float
    """The average number of requests seen during this period"""


class Features(BaseModel):
    traffic_stats: Optional[FeaturesTrafficStats] = None


class DiscoveryOperation(BaseModel):
    id: str
    """UUID."""

    endpoint: str
    """
    The endpoint which can contain path parameter templates in curly braces, each
    will be replaced from left to right with {varN}, starting with {var1}, during
    insertion. This will further be Cloudflare-normalized upon insertion. See:
    https://developers.cloudflare.com/rules/normalization/how-it-works/.
    """

    host: str
    """RFC3986-compliant host."""

    last_updated: datetime

    method: Literal["GET", "POST", "HEAD", "OPTIONS", "PUT", "DELETE", "CONNECT", "PATCH", "TRACE"]
    """The HTTP method used to access the endpoint."""

    origin: List[Literal["ML", "SessionIdentifier", "LabelDiscovery"]]
    """API discovery engine(s) that discovered this operation"""

    state: Literal["review", "saved", "ignored"]
    """State of operation in API Discovery

    - `review` - Operation is not saved into API Shield Endpoint Management
    - `saved` - Operation is saved into API Shield Endpoint Management
    - `ignored` - Operation is marked as ignored
    """

    features: Optional[Features] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["HostnameRouteCreateParams"]


class HostnameRouteCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Cloudflare account ID"""

    comment: str
    """An optional description of the hostname route."""

    hostname: str
    """The hostname of the route."""

    tunnel_id: str
    """UUID of the tunnel."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["CustomCertificateListParams"]


class CustomCertificateListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    match: Literal["any", "all"]
    """Whether to match all search requirements or at least one (any)."""

    page: float
    """Page number of paginated results."""

    per_page: float
    """Number of zones per page."""

    status: Literal["active", "expired", "deleted", "pending", "initializing"]
    """Status of the zone's custom SSL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import Literal, TypeAlias

from ...._models import BaseModel
from ...api_gateway.message import Message

__all__ = [
    "CredentialUpdateResponse",
    "Key",
    "KeyAPIShieldCredentialsJWTKeyRSA",
    "KeyAPIShieldCredentialsJWTKeyEcEs256",
    "KeyAPIShieldCredentialsJWTKeyEcEs384",
]


class KeyAPIShieldCredentialsJWTKeyRSA(BaseModel):
    """JSON representation of an RSA key."""

    alg: Literal["RS256", "RS384", "RS512", "PS256", "PS384", "PS512"]
    """Algorithm"""

    e: str
    """RSA exponent"""

    kid: str
    """Key ID"""

    kty: Literal["RSA"]
    """Key Type"""

    n: str
    """RSA modulus"""


class KeyAPIShieldCredentialsJWTKeyEcEs256(BaseModel):
    """JSON representation of an ES256 key"""

    alg: Literal["ES256"]
    """Algorithm"""

    crv: Literal["P-256"]
    """Curve"""

    kid: str
    """Key ID"""

    kty: Literal["EC"]
    """Key Type"""

    x: str
    """X EC coordinate"""

    y: str
    """Y EC coordinate"""


class KeyAPIShieldCredentialsJWTKeyEcEs384(BaseModel):
    """JSON representation of an ES384 key"""

    alg: Literal["ES384"]
    """Algorithm"""

    crv: Literal["P-384"]
    """Curve"""

    kid: str
    """Key ID"""

    kty: Literal["EC"]
    """Key Type"""

    x: str
    """X EC coordinate"""

    y: str
    """Y EC coordinate"""


Key: TypeAlias = Union[
    KeyAPIShieldCredentialsJWTKeyRSA, KeyAPIShieldCredentialsJWTKeyEcEs256, KeyAPIShieldCredentialsJWTKeyEcEs384
]


class CredentialUpdateResponse(BaseModel):
    errors: Message

    keys: List[Key]

    messages: Message

    success: Literal[True]
    """Whether the API call was successful."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["RouteStatsResponse", "Meta", "Stats"]


class Meta(BaseModel):
    data_time: str

    query_time: str

    total_peers: int


class Stats(BaseModel):
    distinct_origins: int

    distinct_origins_ipv4: int

    distinct_origins_ipv6: int

    distinct_prefixes: int

    distinct_prefixes_ipv4: int

    distinct_prefixes_ipv6: int

    routes_invalid: int

    routes_invalid_ipv4: int

    routes_invalid_ipv6: int

    routes_total: int

    routes_total_ipv4: int

    routes_total_ipv6: int

    routes_unknown: int

    routes_unknown_ipv4: int

    routes_unknown_ipv6: int

    routes_valid: int

    routes_valid_ipv4: int

    routes_valid_ipv6: int


class RouteStatsResponse(BaseModel):
    meta: Meta

    stats: Stats



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel

__all__ = [
    "AIRunResponse",
    "TextClassification",
    "Audio",
    "TextEmbeddings",
    "AutomaticSpeechRecognition",
    "AutomaticSpeechRecognitionWord",
    "ImageClassification",
    "ObjectDetection",
    "ObjectDetectionBox",
    "UnionMember8",
    "UnionMember8ToolCall",
    "UnionMember8Usage",
    "Translation",
    "Summarization",
    "ImageToText",
    "ImageTextToText",
    "MultimodalEmbeddings",
]


class TextClassification(BaseModel):
    label: Optional[str] = None
    """The classification label assigned to the text (e.g., 'POSITIVE' or 'NEGATIVE')"""

    score: Optional[float] = None
    """
    Confidence score indicating the likelihood that the text belongs to the
    specified label
    """


class Audio(BaseModel):
    audio: Optional[str] = None
    """The generated audio in MP3 format, base64-encoded"""


class TextEmbeddings(BaseModel):
    data: Optional[List[List[float]]] = None
    """Embeddings of the requested text values"""

    shape: Optional[List[float]] = None


class AutomaticSpeechRecognitionWord(BaseModel):
    end: Optional[float] = None
    """The ending second when the word completes"""

    start: Optional[float] = None
    """The second this word begins in the recording"""

    word: Optional[str] = None


class AutomaticSpeechRecognition(BaseModel):
    text: str
    """The transcription"""

    vtt: Optional[str] = None

    word_count: Optional[float] = None

    words: Optional[List[AutomaticSpeechRecognitionWord]] = None


class ImageClassification(BaseModel):
    label: Optional[str] = None
    """The predicted category or class for the input image based on analysis"""

    score: Optional[float] = None
    """
    A confidence value, between 0 and 1, indicating how certain the model is about
    the predicted label
    """


class ObjectDetectionBox(BaseModel):
    """Coordinates defining the bounding box around the detected object"""

    xmax: Optional[float] = None
    """The x-coordinate of the bottom-right corner of the bounding box"""

    xmin: Optional[float] = None
    """The x-coordinate of the top-left corner of the bounding box"""

    ymax: Optional[float] = None
    """The y-coordinate of the bottom-right corner of the bounding box"""

    ymin: Optional[float] = None
    """The y-coordinate of the top-left corner of the bounding box"""


class ObjectDetection(BaseModel):
    box: Optional[ObjectDetectionBox] = None
    """Coordinates defining the bounding box around the detected object"""

    label: Optional[str] = None
    """The class label or name of the detected object"""

    score: Optional[float] = None
    """Confidence score indicating the likelihood that the detection is correct"""


class UnionMember8ToolCall(BaseModel):
    arguments: Optional[object] = None
    """The arguments passed to be passed to the tool call request"""

    name: Optional[str] = None
    """The name of the tool to be called"""


class UnionMember8Usage(BaseModel):
    """Usage statistics for the inference request"""

    completion_tokens: Optional[float] = None
    """Total number of tokens in output"""

    prompt_tokens: Optional[float] = None
    """Total number of tokens in input"""

    total_tokens: Optional[float] = None
    """Total number of input and output tokens"""


class UnionMember8(BaseModel):
    response: str
    """The generated text response from the model"""

    tool_calls: Optional[List[UnionMember8ToolCall]] = None
    """An array of tool calls requests made during the response generation"""

    usage: Optional[UnionMember8Usage] = None
    """Usage statistics for the inference request"""


class Translation(BaseModel):
    translated_text: Optional[str] = None
    """The translated text in the target language"""


class Summarization(BaseModel):
    summary: Optional[str] = None
    """The summarized version of the input text"""


class ImageToText(BaseModel):
    description: Optional[str] = None


class ImageTextToText(BaseModel):
    description: Optional[str] = None


class MultimodalEmbeddings(BaseModel):
    data: Optional[List[List[float]]] = None

    shape: Optional[List[float]] = None


AIRunResponse: TypeAlias = Union[
    List[TextClassification],
    object,
    Audio,
    object,
    TextEmbeddings,
    AutomaticSpeechRecognition,
    List[ImageClassification],
    List[ObjectDetection],
    UnionMember8,
    object,
    Translation,
    Summarization,
    ImageToText,
    ImageTextToText,
    MultimodalEmbeddings,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .failed_logins import (
    FailedLoginsResource,
    AsyncFailedLoginsResource,
    FailedLoginsResourceWithRawResponse,
    AsyncFailedLoginsResourceWithRawResponse,
    FailedLoginsResourceWithStreamingResponse,
    AsyncFailedLoginsResourceWithStreamingResponse,
)
from .active_sessions import (
    ActiveSessionsResource,
    AsyncActiveSessionsResource,
    ActiveSessionsResourceWithRawResponse,
    AsyncActiveSessionsResourceWithRawResponse,
    ActiveSessionsResourceWithStreamingResponse,
    AsyncActiveSessionsResourceWithStreamingResponse,
)
from .last_seen_identity import (
    LastSeenIdentityResource,
    AsyncLastSeenIdentityResource,
    LastSeenIdentityResourceWithRawResponse,
    AsyncLastSeenIdentityResourceWithRawResponse,
    LastSeenIdentityResourceWithStreamingResponse,
    AsyncLastSeenIdentityResourceWithStreamingResponse,
)

__all__ = [
    "ActiveSessionsResource",
    "AsyncActiveSessionsResource",
    "ActiveSessionsResourceWithRawResponse",
    "AsyncActiveSessionsResourceWithRawResponse",
    "ActiveSessionsResourceWithStreamingResponse",
    "AsyncActiveSessionsResourceWithStreamingResponse",
    "LastSeenIdentityResource",
    "AsyncLastSeenIdentityResource",
    "LastSeenIdentityResourceWithRawResponse",
    "AsyncLastSeenIdentityResourceWithRawResponse",
    "LastSeenIdentityResourceWithStreamingResponse",
    "AsyncLastSeenIdentityResourceWithStreamingResponse",
    "FailedLoginsResource",
    "AsyncFailedLoginsResource",
    "FailedLoginsResourceWithRawResponse",
    "AsyncFailedLoginsResourceWithRawResponse",
    "FailedLoginsResourceWithStreamingResponse",
    "AsyncFailedLoginsResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["IPProfileUpdateParams"]


class IPProfileUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    description: str
    """An optional description of the Device IP profile."""

    enabled: bool
    """Whether the Device IP profile is enabled."""

    match: str
    """The wirefilter expression to match registrations.

    Available values: "identity.name", "identity.email", "identity.groups.id",
    "identity.groups.name", "identity.groups.email", "identity.saml_attributes".
    """

    name: str
    """A user-friendly name for the Device IP profile."""

    precedence: int
    """The precedence of the Device IP profile.

    Lower values indicate higher precedence. Device IP profile will be evaluated in
    ascending order of this field.
    """

    subnet_id: str
    """The ID of the Subnet."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["EventNotificationListResponse", "Queue", "QueueRule"]


class QueueRule(BaseModel):
    actions: List[Literal["PutObject", "CopyObject", "DeleteObject", "CompleteMultipartUpload", "LifecycleDeletion"]]
    """Array of R2 object actions that will trigger notifications."""

    created_at: Optional[str] = FieldInfo(alias="createdAt", default=None)
    """Timestamp when the rule was created."""

    description: Optional[str] = None
    """
    A description that can be used to identify the event notification rule after
    creation.
    """

    prefix: Optional[str] = None
    """Notifications will be sent only for objects with this prefix."""

    rule_id: Optional[str] = FieldInfo(alias="ruleId", default=None)
    """Rule ID."""

    suffix: Optional[str] = None
    """Notifications will be sent only for objects with this suffix."""


class Queue(BaseModel):
    queue_id: Optional[str] = FieldInfo(alias="queueId", default=None)
    """Queue ID."""

    queue_name: Optional[str] = FieldInfo(alias="queueName", default=None)
    """Name of the queue."""

    rules: Optional[List[QueueRule]] = None


class EventNotificationListResponse(BaseModel):
    bucket_name: Optional[str] = FieldInfo(alias="bucketName", default=None)
    """Name of the bucket."""

    queues: Optional[List[Queue]] = None
    """List of queues associated with the bucket."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["SubscriptionComponent"]


class SubscriptionComponent(BaseModel):
    """A component value for a subscription."""

    default: Optional[float] = None
    """The default amount assigned."""

    name: Optional[str] = None
    """The name of the component value."""

    price: Optional[float] = None
    """The unit price for the component value."""

    value: Optional[float] = None
    """The amount of the component value assigned."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DomainUpdateParams"]


class DomainUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    auto_renew: bool
    """
    Auto-renew controls whether subscription is automatically renewed upon domain
    expiration.
    """

    locked: bool
    """Shows whether a registrar lock is in place for a domain."""

    privacy: bool
    """Privacy option controls redacting WHOIS information."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.radar.bgp import ip_timeseries_params
from ....types.radar.bgp.ip_timeseries_response import IPTimeseriesResponse

__all__ = ["IPsResource", "AsyncIPsResource"]


class IPsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IPsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IPsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IPsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IPsResourceWithStreamingResponse(self)

    def timeseries(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        include_delay: bool | Omit = omit,
        ip_version: List[Literal["IPv4", "IPv6"]] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPTimeseriesResponse:
        """
        Retrieves time series data for the announced IP space count, represented as the
        number of IPv4 /24s and IPv6 /48s, for a given ASN.

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          include_delay: Includes data delay meta information.

          ip_version: Filters results by IP version (Ipv4 vs. IPv6).

          location: Filters results by location. Specify a comma-separated list of alpha-2 location
              codes.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/bgp/ips/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "include_delay": include_delay,
                        "ip_version": ip_version,
                        "location": location,
                        "name": name,
                    },
                    ip_timeseries_params.IPTimeseriesParams,
                ),
                post_parser=ResultWrapper[IPTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[IPTimeseriesResponse], ResultWrapper[IPTimeseriesResponse]),
        )


class AsyncIPsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIPsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIPsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIPsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIPsResourceWithStreamingResponse(self)

    async def timeseries(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        include_delay: bool | Omit = omit,
        ip_version: List[Literal["IPv4", "IPv6"]] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPTimeseriesResponse:
        """
        Retrieves time series data for the announced IP space count, represented as the
        number of IPv4 /24s and IPv6 /48s, for a given ASN.

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          include_delay: Includes data delay meta information.

          ip_version: Filters results by IP version (Ipv4 vs. IPv6).

          location: Filters results by location. Specify a comma-separated list of alpha-2 location
              codes.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/bgp/ips/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "include_delay": include_delay,
                        "ip_version": ip_version,
                        "location": location,
                        "name": name,
                    },
                    ip_timeseries_params.IPTimeseriesParams,
                ),
                post_parser=ResultWrapper[IPTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[IPTimeseriesResponse], ResultWrapper[IPTimeseriesResponse]),
        )


class IPsResourceWithRawResponse:
    def __init__(self, ips: IPsResource) -> None:
        self._ips = ips

        self.timeseries = to_raw_response_wrapper(
            ips.timeseries,
        )


class AsyncIPsResourceWithRawResponse:
    def __init__(self, ips: AsyncIPsResource) -> None:
        self._ips = ips

        self.timeseries = async_to_raw_response_wrapper(
            ips.timeseries,
        )


class IPsResourceWithStreamingResponse:
    def __init__(self, ips: IPsResource) -> None:
        self._ips = ips

        self.timeseries = to_streamed_response_wrapper(
            ips.timeseries,
        )


class AsyncIPsResourceWithStreamingResponse:
    def __init__(self, ips: AsyncIPsResource) -> None:
        self._ips = ips

        self.timeseries = async_to_streamed_response_wrapper(
            ips.timeseries,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .operation import (
    OperationResource,
    AsyncOperationResource,
    OperationResourceWithRawResponse,
    AsyncOperationResourceWithRawResponse,
    OperationResourceWithStreamingResponse,
    AsyncOperationResourceWithStreamingResponse,
)
from .resources import (
    ResourcesResource,
    AsyncResourcesResource,
    ResourcesResourceWithRawResponse,
    AsyncResourcesResourceWithRawResponse,
    ResourcesResourceWithStreamingResponse,
    AsyncResourcesResourceWithStreamingResponse,
)

__all__ = [
    "OperationResource",
    "AsyncOperationResource",
    "OperationResourceWithRawResponse",
    "AsyncOperationResourceWithRawResponse",
    "OperationResourceWithStreamingResponse",
    "AsyncOperationResourceWithStreamingResponse",
    "ResourcesResource",
    "AsyncResourcesResource",
    "ResourcesResourceWithRawResponse",
    "AsyncResourcesResourceWithRawResponse",
    "ResourcesResourceWithStreamingResponse",
    "AsyncResourcesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.turnstile import (
    widget_list_params,
    widget_create_params,
    widget_update_params,
    widget_rotate_secret_params,
)
from ...types.turnstile.widget import Widget
from ...types.turnstile.widget_domain import WidgetDomain
from ...types.turnstile.widget_list_response import WidgetListResponse

__all__ = ["WidgetsResource", "AsyncWidgetsResource"]


class WidgetsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WidgetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return WidgetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WidgetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return WidgetsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        domains: SequenceNotStr[WidgetDomain],
        mode: Literal["non-interactive", "invisible", "managed"],
        name: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        filter: str | Omit = omit,
        order: Literal["id", "sitekey", "name", "created_on", "modified_on"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        bot_fight_mode: bool | Omit = omit,
        clearance_level: Literal["no_clearance", "jschallenge", "managed", "interactive"] | Omit = omit,
        ephemeral_id: bool | Omit = omit,
        offlabel: bool | Omit = omit,
        region: Literal["world", "china"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Widget]:
        """
        Lists challenge widgets.

        Args:
          account_id: Identifier

          mode: Widget Mode

          name: Human readable widget name. Not unique. Cloudflare suggests that you set this to
              a meaningful string to make it easier to identify your widget, and where it is
              used.

          direction: Direction to order widgets.

          filter:
              Filter widgets by field using case-insensitive substring matching. Format:
              `field:value`

              Supported fields:

              - `name` - Filter by widget name (e.g., `filter=name:login-form`)
              - `sitekey` - Filter by sitekey (e.g., `filter=sitekey:0x4AAA`)

              Returns 400 Bad Request if the field is unsupported or format is invalid. An
              empty filter value returns all results.

          order: Field to order widgets by.

          page: Page number of paginated results.

          per_page: Number of items per page.

          bot_fight_mode: If bot_fight_mode is set to `true`, Cloudflare issues computationally expensive
              challenges in response to malicious bots (ENT only).

          clearance_level: If Turnstile is embedded on a Cloudflare site and the widget should grant
              challenge clearance, this setting can determine the clearance level to be set

          ephemeral_id: Return the Ephemeral ID in /siteverify (ENT only).

          offlabel: Do not show any Cloudflare branding on the widget (ENT only).

          region: Region where this widget can be used. This cannot be changed after creation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/challenges/widgets", account_id=account_id),
            body=maybe_transform(
                {
                    "domains": domains,
                    "mode": mode,
                    "name": name,
                    "bot_fight_mode": bot_fight_mode,
                    "clearance_level": clearance_level,
                    "ephemeral_id": ephemeral_id,
                    "offlabel": offlabel,
                    "region": region,
                },
                widget_create_params.WidgetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "filter": filter,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    widget_create_params.WidgetCreateParams,
                ),
                post_parser=ResultWrapper[Optional[Widget]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Widget]], ResultWrapper[Widget]),
        )

    def update(
        self,
        sitekey: str,
        *,
        account_id: str,
        domains: SequenceNotStr[WidgetDomain],
        mode: Literal["non-interactive", "invisible", "managed"],
        name: str,
        bot_fight_mode: bool | Omit = omit,
        clearance_level: Literal["no_clearance", "jschallenge", "managed", "interactive"] | Omit = omit,
        ephemeral_id: bool | Omit = omit,
        offlabel: bool | Omit = omit,
        region: Literal["world", "china"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Widget]:
        """
        Update the configuration of a widget.

        Args:
          account_id: Identifier

          sitekey: Widget item identifier tag.

          mode: Widget Mode

          name: Human readable widget name. Not unique. Cloudflare suggests that you set this to
              a meaningful string to make it easier to identify your widget, and where it is
              used.

          bot_fight_mode: If bot_fight_mode is set to `true`, Cloudflare issues computationally expensive
              challenges in response to malicious bots (ENT only).

          clearance_level: If Turnstile is embedded on a Cloudflare site and the widget should grant
              challenge clearance, this setting can determine the clearance level to be set

          ephemeral_id: Return the Ephemeral ID in /siteverify (ENT only).

          offlabel: Do not show any Cloudflare branding on the widget (ENT only).

          region: Region where this widget can be used. This cannot be changed after creation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sitekey:
            raise ValueError(f"Expected a non-empty value for `sitekey` but received {sitekey!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/challenges/widgets/{sitekey}", account_id=account_id, sitekey=sitekey
            ),
            body=maybe_transform(
                {
                    "domains": domains,
                    "mode": mode,
                    "name": name,
                    "bot_fight_mode": bot_fight_mode,
                    "clearance_level": clearance_level,
                    "ephemeral_id": ephemeral_id,
                    "offlabel": offlabel,
                    "region": region,
                },
                widget_update_params.WidgetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Widget]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Widget]], ResultWrapper[Widget]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        filter: str | Omit = omit,
        order: Literal["id", "sitekey", "name", "created_on", "modified_on"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[WidgetListResponse]:
        """
        Lists all turnstile widgets of an account.

        Args:
          account_id: Identifier

          direction: Direction to order widgets.

          filter:
              Filter widgets by field using case-insensitive substring matching. Format:
              `field:value`

              Supported fields:

              - `name` - Filter by widget name (e.g., `filter=name:login-form`)
              - `sitekey` - Filter by sitekey (e.g., `filter=sitekey:0x4AAA`)

              Returns 400 Bad Request if the field is unsupported or format is invalid. An
              empty filter value returns all results.

          order: Field to order widgets by.

          page: Page number of paginated results.

          per_page: Number of items per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/challenges/widgets", account_id=account_id),
            page=SyncV4PagePaginationArray[WidgetListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "filter": filter,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    widget_list_params.WidgetListParams,
                ),
            ),
            model=WidgetListResponse,
        )

    def delete(
        self,
        sitekey: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Widget]:
        """
        Destroy a Turnstile Widget.

        Args:
          account_id: Identifier

          sitekey: Widget item identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sitekey:
            raise ValueError(f"Expected a non-empty value for `sitekey` but received {sitekey!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/challenges/widgets/{sitekey}", account_id=account_id, sitekey=sitekey
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Widget]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Widget]], ResultWrapper[Widget]),
        )

    def get(
        self,
        sitekey: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Widget]:
        """
        Show a single challenge widget configuration.

        Args:
          account_id: Identifier

          sitekey: Widget item identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sitekey:
            raise ValueError(f"Expected a non-empty value for `sitekey` but received {sitekey!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/challenges/widgets/{sitekey}", account_id=account_id, sitekey=sitekey
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Widget]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Widget]], ResultWrapper[Widget]),
        )

    def rotate_secret(
        self,
        sitekey: str,
        *,
        account_id: str,
        invalidate_immediately: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Widget]:
        """Generate a new secret key for this widget.

        If `invalidate_immediately` is set to
        `false`, the previous secret remains valid for 2 hours.

        Note that secrets cannot be rotated again during the grace period.

        Args:
          account_id: Identifier

          sitekey: Widget item identifier tag.

          invalidate_immediately: If `invalidate_immediately` is set to `false`, the previous secret will remain
              valid for two hours. Otherwise, the secret is immediately invalidated, and
              requests using it will be rejected.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sitekey:
            raise ValueError(f"Expected a non-empty value for `sitekey` but received {sitekey!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/challenges/widgets/{sitekey}/rotate_secret",
                account_id=account_id,
                sitekey=sitekey,
            ),
            body=maybe_transform(
                {"invalidate_immediately": invalidate_immediately}, widget_rotate_secret_params.WidgetRotateSecretParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Widget]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Widget]], ResultWrapper[Widget]),
        )


class AsyncWidgetsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWidgetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWidgetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWidgetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncWidgetsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        domains: SequenceNotStr[WidgetDomain],
        mode: Literal["non-interactive", "invisible", "managed"],
        name: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        filter: str | Omit = omit,
        order: Literal["id", "sitekey", "name", "created_on", "modified_on"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        bot_fight_mode: bool | Omit = omit,
        clearance_level: Literal["no_clearance", "jschallenge", "managed", "interactive"] | Omit = omit,
        ephemeral_id: bool | Omit = omit,
        offlabel: bool | Omit = omit,
        region: Literal["world", "china"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Widget]:
        """
        Lists challenge widgets.

        Args:
          account_id: Identifier

          mode: Widget Mode

          name: Human readable widget name. Not unique. Cloudflare suggests that you set this to
              a meaningful string to make it easier to identify your widget, and where it is
              used.

          direction: Direction to order widgets.

          filter:
              Filter widgets by field using case-insensitive substring matching. Format:
              `field:value`

              Supported fields:

              - `name` - Filter by widget name (e.g., `filter=name:login-form`)
              - `sitekey` - Filter by sitekey (e.g., `filter=sitekey:0x4AAA`)

              Returns 400 Bad Request if the field is unsupported or format is invalid. An
              empty filter value returns all results.

          order: Field to order widgets by.

          page: Page number of paginated results.

          per_page: Number of items per page.

          bot_fight_mode: If bot_fight_mode is set to `true`, Cloudflare issues computationally expensive
              challenges in response to malicious bots (ENT only).

          clearance_level: If Turnstile is embedded on a Cloudflare site and the widget should grant
              challenge clearance, this setting can determine the clearance level to be set

          ephemeral_id: Return the Ephemeral ID in /siteverify (ENT only).

          offlabel: Do not show any Cloudflare branding on the widget (ENT only).

          region: Region where this widget can be used. This cannot be changed after creation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/challenges/widgets", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "domains": domains,
                    "mode": mode,
                    "name": name,
                    "bot_fight_mode": bot_fight_mode,
                    "clearance_level": clearance_level,
                    "ephemeral_id": ephemeral_id,
                    "offlabel": offlabel,
                    "region": region,
                },
                widget_create_params.WidgetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "direction": direction,
                        "filter": filter,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    widget_create_params.WidgetCreateParams,
                ),
                post_parser=ResultWrapper[Optional[Widget]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Widget]], ResultWrapper[Widget]),
        )

    async def update(
        self,
        sitekey: str,
        *,
        account_id: str,
        domains: SequenceNotStr[WidgetDomain],
        mode: Literal["non-interactive", "invisible", "managed"],
        name: str,
        bot_fight_mode: bool | Omit = omit,
        clearance_level: Literal["no_clearance", "jschallenge", "managed", "interactive"] | Omit = omit,
        ephemeral_id: bool | Omit = omit,
        offlabel: bool | Omit = omit,
        region: Literal["world", "china"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Widget]:
        """
        Update the configuration of a widget.

        Args:
          account_id: Identifier

          sitekey: Widget item identifier tag.

          mode: Widget Mode

          name: Human readable widget name. Not unique. Cloudflare suggests that you set this to
              a meaningful string to make it easier to identify your widget, and where it is
              used.

          bot_fight_mode: If bot_fight_mode is set to `true`, Cloudflare issues computationally expensive
              challenges in response to malicious bots (ENT only).

          clearance_level: If Turnstile is embedded on a Cloudflare site and the widget should grant
              challenge clearance, this setting can determine the clearance level to be set

          ephemeral_id: Return the Ephemeral ID in /siteverify (ENT only).

          offlabel: Do not show any Cloudflare branding on the widget (ENT only).

          region: Region where this widget can be used. This cannot be changed after creation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sitekey:
            raise ValueError(f"Expected a non-empty value for `sitekey` but received {sitekey!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/challenges/widgets/{sitekey}", account_id=account_id, sitekey=sitekey
            ),
            body=await async_maybe_transform(
                {
                    "domains": domains,
                    "mode": mode,
                    "name": name,
                    "bot_fight_mode": bot_fight_mode,
                    "clearance_level": clearance_level,
                    "ephemeral_id": ephemeral_id,
                    "offlabel": offlabel,
                    "region": region,
                },
                widget_update_params.WidgetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Widget]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Widget]], ResultWrapper[Widget]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        filter: str | Omit = omit,
        order: Literal["id", "sitekey", "name", "created_on", "modified_on"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[WidgetListResponse, AsyncV4PagePaginationArray[WidgetListResponse]]:
        """
        Lists all turnstile widgets of an account.

        Args:
          account_id: Identifier

          direction: Direction to order widgets.

          filter:
              Filter widgets by field using case-insensitive substring matching. Format:
              `field:value`

              Supported fields:

              - `name` - Filter by widget name (e.g., `filter=name:login-form`)
              - `sitekey` - Filter by sitekey (e.g., `filter=sitekey:0x4AAA`)

              Returns 400 Bad Request if the field is unsupported or format is invalid. An
              empty filter value returns all results.

          order: Field to order widgets by.

          page: Page number of paginated results.

          per_page: Number of items per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/challenges/widgets", account_id=account_id),
            page=AsyncV4PagePaginationArray[WidgetListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "filter": filter,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    widget_list_params.WidgetListParams,
                ),
            ),
            model=WidgetListResponse,
        )

    async def delete(
        self,
        sitekey: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Widget]:
        """
        Destroy a Turnstile Widget.

        Args:
          account_id: Identifier

          sitekey: Widget item identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sitekey:
            raise ValueError(f"Expected a non-empty value for `sitekey` but received {sitekey!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/challenges/widgets/{sitekey}", account_id=account_id, sitekey=sitekey
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Widget]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Widget]], ResultWrapper[Widget]),
        )

    async def get(
        self,
        sitekey: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Widget]:
        """
        Show a single challenge widget configuration.

        Args:
          account_id: Identifier

          sitekey: Widget item identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sitekey:
            raise ValueError(f"Expected a non-empty value for `sitekey` but received {sitekey!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/challenges/widgets/{sitekey}", account_id=account_id, sitekey=sitekey
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Widget]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Widget]], ResultWrapper[Widget]),
        )

    async def rotate_secret(
        self,
        sitekey: str,
        *,
        account_id: str,
        invalidate_immediately: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Widget]:
        """Generate a new secret key for this widget.

        If `invalidate_immediately` is set to
        `false`, the previous secret remains valid for 2 hours.

        Note that secrets cannot be rotated again during the grace period.

        Args:
          account_id: Identifier

          sitekey: Widget item identifier tag.

          invalidate_immediately: If `invalidate_immediately` is set to `false`, the previous secret will remain
              valid for two hours. Otherwise, the secret is immediately invalidated, and
              requests using it will be rejected.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sitekey:
            raise ValueError(f"Expected a non-empty value for `sitekey` but received {sitekey!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/challenges/widgets/{sitekey}/rotate_secret",
                account_id=account_id,
                sitekey=sitekey,
            ),
            body=await async_maybe_transform(
                {"invalidate_immediately": invalidate_immediately}, widget_rotate_secret_params.WidgetRotateSecretParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Widget]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Widget]], ResultWrapper[Widget]),
        )


class WidgetsResourceWithRawResponse:
    def __init__(self, widgets: WidgetsResource) -> None:
        self._widgets = widgets

        self.create = to_raw_response_wrapper(
            widgets.create,
        )
        self.update = to_raw_response_wrapper(
            widgets.update,
        )
        self.list = to_raw_response_wrapper(
            widgets.list,
        )
        self.delete = to_raw_response_wrapper(
            widgets.delete,
        )
        self.get = to_raw_response_wrapper(
            widgets.get,
        )
        self.rotate_secret = to_raw_response_wrapper(
            widgets.rotate_secret,
        )


class AsyncWidgetsResourceWithRawResponse:
    def __init__(self, widgets: AsyncWidgetsResource) -> None:
        self._widgets = widgets

        self.create = async_to_raw_response_wrapper(
            widgets.create,
        )
        self.update = async_to_raw_response_wrapper(
            widgets.update,
        )
        self.list = async_to_raw_response_wrapper(
            widgets.list,
        )
        self.delete = async_to_raw_response_wrapper(
            widgets.delete,
        )
        self.get = async_to_raw_response_wrapper(
            widgets.get,
        )
        self.rotate_secret = async_to_raw_response_wrapper(
            widgets.rotate_secret,
        )


class WidgetsResourceWithStreamingResponse:
    def __init__(self, widgets: WidgetsResource) -> None:
        self._widgets = widgets

        self.create = to_streamed_response_wrapper(
            widgets.create,
        )
        self.update = to_streamed_response_wrapper(
            widgets.update,
        )
        self.list = to_streamed_response_wrapper(
            widgets.list,
        )
        self.delete = to_streamed_response_wrapper(
            widgets.delete,
        )
        self.get = to_streamed_response_wrapper(
            widgets.get,
        )
        self.rotate_secret = to_streamed_response_wrapper(
            widgets.rotate_secret,
        )


class AsyncWidgetsResourceWithStreamingResponse:
    def __init__(self, widgets: AsyncWidgetsResource) -> None:
        self._widgets = widgets

        self.create = async_to_streamed_response_wrapper(
            widgets.create,
        )
        self.update = async_to_streamed_response_wrapper(
            widgets.update,
        )
        self.list = async_to_streamed_response_wrapper(
            widgets.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            widgets.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            widgets.get,
        )
        self.rotate_secret = async_to_streamed_response_wrapper(
            widgets.rotate_secret,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from .fields import (
    FieldsResource,
    AsyncFieldsResource,
    FieldsResourceWithRawResponse,
    AsyncFieldsResourceWithRawResponse,
    FieldsResourceWithStreamingResponse,
    AsyncFieldsResourceWithStreamingResponse,
)
from .datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)

__all__ = [
    "FieldsResource",
    "AsyncFieldsResource",
    "FieldsResourceWithRawResponse",
    "AsyncFieldsResourceWithRawResponse",
    "FieldsResourceWithStreamingResponse",
    "AsyncFieldsResourceWithStreamingResponse",
    "JobsResource",
    "AsyncJobsResource",
    "JobsResourceWithRawResponse",
    "AsyncJobsResourceWithRawResponse",
    "JobsResourceWithStreamingResponse",
    "AsyncJobsResourceWithStreamingResponse",
    "DatasetsResource",
    "AsyncDatasetsResource",
    "DatasetsResourceWithRawResponse",
    "AsyncDatasetsResourceWithRawResponse",
    "DatasetsResourceWithStreamingResponse",
    "AsyncDatasetsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["PacfileGetResponse"]


class PacfileGetResponse(BaseModel):
    id: Optional[str] = None

    contents: Optional[str] = None
    """Actual contents of the PAC file"""

    created_at: Optional[datetime] = None

    description: Optional[str] = None
    """Detailed description of the PAC file."""

    name: Optional[str] = None
    """Name of the PAC file."""

    slug: Optional[str] = None
    """URL-friendly version of the PAC file name."""

    updated_at: Optional[datetime] = None

    url: Optional[str] = None
    """Unique URL to download the PAC file."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Literal, TypeAlias

from ...._models import BaseModel

__all__ = [
    "BulkOperationGetResponse",
    "ListsBulkOperationPendingOrRunning",
    "ListsBulkOperationCompleted",
    "ListsBulkOperationFailed",
]


class ListsBulkOperationPendingOrRunning(BaseModel):
    id: str
    """The unique operation ID of the asynchronous action."""

    status: Literal["pending", "running"]
    """The current status of the asynchronous operation."""


class ListsBulkOperationCompleted(BaseModel):
    id: str
    """The unique operation ID of the asynchronous action."""

    completed: str
    """The RFC 3339 timestamp of when the operation was completed."""

    status: Literal["completed"]
    """The current status of the asynchronous operation."""


class ListsBulkOperationFailed(BaseModel):
    id: str
    """The unique operation ID of the asynchronous action."""

    completed: str
    """The RFC 3339 timestamp of when the operation was completed."""

    error: str
    """A message describing the error when the status is `failed`."""

    status: Literal["failed"]
    """The current status of the asynchronous operation."""


BulkOperationGetResponse: TypeAlias = Union[
    ListsBulkOperationPendingOrRunning, ListsBulkOperationCompleted, ListsBulkOperationFailed
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["ThreatEventCreateParams", "Raw", "Indicator"]


class ThreatEventCreateParams(TypedDict, total=False):
    path_account_id: Required[Annotated[str, PropertyInfo(alias="account_id")]]
    """Account ID."""

    category: Required[str]

    date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    event: Required[str]

    raw: Required[Raw]

    tlp: Required[str]

    body_account_id: Annotated[float, PropertyInfo(alias="accountId")]

    attacker: Optional[str]

    attacker_country: Annotated[str, PropertyInfo(alias="attackerCountry")]

    dataset_id: Annotated[str, PropertyInfo(alias="datasetId")]

    indicator: str

    indicators: Iterable[Indicator]
    """Array of indicators for this event.

    Supports multiple indicators per event for complex scenarios.
    """

    indicator_type: Annotated[str, PropertyInfo(alias="indicatorType")]

    insight: str

    tags: SequenceNotStr[str]

    target_country: Annotated[str, PropertyInfo(alias="targetCountry")]

    target_industry: Annotated[str, PropertyInfo(alias="targetIndustry")]


class Raw(TypedDict, total=False):
    data: Required[Optional[Dict[str, object]]]

    source: str

    tlp: str


class Indicator(TypedDict, total=False):
    indicator_type: Required[Annotated[str, PropertyInfo(alias="indicatorType")]]
    """The type of indicator (e.g., DOMAIN, IP, JA3, HASH)"""

    value: Required[str]
    """The indicator value (e.g., domain name, IP address, hash)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["WorkflowUpdateParams", "Limits"]


class WorkflowUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    class_name: Required[str]

    script_name: Required[str]

    limits: Limits


class Limits(TypedDict, total=False):
    steps: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .new_version import NewVersion as NewVersion
from .upload_edit_params import UploadEditParams as UploadEditParams
from .version_create_params import VersionCreateParams as VersionCreateParams
from .version_create_response import VersionCreateResponse as VersionCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ScanListParams"]


class ScanListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    q: str
    """Filter scans"""

    size: int
    """Limit the number of objects in the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["LeakedCredentialCheckCreateResponse"]


class LeakedCredentialCheckCreateResponse(BaseModel):
    """Defines the overall status for Leaked Credential Checks."""

    enabled: Optional[bool] = None
    """Determines whether or not Leaked Credential Checks are enabled."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .authenticated_origin_pull import AuthenticatedOriginPull

__all__ = ["HostnameUpdateResponse"]


class HostnameUpdateResponse(AuthenticatedOriginPull):
    id: Optional[str] = None
    """Identifier."""

    cert_id: Optional[str] = None  # type: ignore
    """Identifier."""

    certificate: Optional[str] = None  # type: ignore
    """The hostname certificate."""

    enabled: Optional[bool] = None  # type: ignore
    """Indicates whether hostname-level authenticated origin pulls is enabled.

    A null value voids the association.
    """

    hostname: Optional[str] = None  # type: ignore
    """
    The hostname on the origin for which the client certificate uploaded will be
    used.
    """

    private_key: Optional[str] = None
    """The hostname certificate's private key."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel
from .catch_all_action import CatchAllAction
from .catch_all_matcher import CatchAllMatcher

__all__ = ["CatchAllUpdateResponse"]


class CatchAllUpdateResponse(BaseModel):
    id: Optional[str] = None
    """Routing rule identifier."""

    actions: Optional[List[CatchAllAction]] = None
    """List actions for the catch-all routing rule."""

    enabled: Optional[Literal[True, False]] = None
    """Routing rule status."""

    matchers: Optional[List[CatchAllMatcher]] = None
    """List of matchers for the catch-all routing rule."""

    name: Optional[str] = None
    """Routing rule name."""

    tag: Optional[str] = None
    """Routing rule tag. (Deprecated, replaced by routing rule identifier)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel
from .setting_value import SettingValue

__all__ = ["Setting"]


class Setting(BaseModel):
    created_at: Optional[datetime] = None
    """This is the time the tls setting was originally created for this hostname."""

    hostname: Optional[str] = None
    """The hostname for which the tls settings are set."""

    status: Optional[str] = None
    """Deployment status for the given tls setting."""

    updated_at: Optional[datetime] = None
    """This is the time the tls setting was updated."""

    value: Optional[SettingValue] = None
    """The TLS setting value.

    The type depends on the `setting_id` used in the request path:

    - `ciphers`: an array of allowed cipher suite strings in BoringSSL format (e.g.,
      `["ECDHE-RSA-AES128-GCM-SHA256", "AES128-GCM-SHA256"]`)
    - `min_tls_version`: a string indicating the minimum TLS version — one of
      `"1.0"`, `"1.1"`, `"1.2"`, or `"1.3"` (e.g., `"1.2"`)
    - `http2`: a string indicating whether HTTP/2 is enabled — `"on"` or `"off"`
      (e.g., `"on"`)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["TracerouteTestPercentilesParams"]


class TracerouteTestPercentilesParams(TypedDict, total=False):
    account_id: Required[str]

    from_: Required[Annotated[str, PropertyInfo(alias="from")]]
    """Start time for the query in ISO (RFC3339 - ISO 8601) format"""

    to: Required[str]
    """End time for the query in ISO (RFC3339 - ISO 8601) format"""

    colo: str
    """Optionally filter result stats to a Cloudflare colo.

    Cannot be used in combination with deviceId param.
    """

    device_id: Annotated[SequenceNotStr[str], PropertyInfo(alias="deviceId")]
    """Optionally filter result stats to a specific device(s).

    Cannot be used in combination with colo param.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ListUpdateParams"]


class ListUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """The Account ID for this resource."""

    description: str
    """An informative summary of the list."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ImpersonationRegistryListParams"]


class ImpersonationRegistryListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    direction: Literal["asc", "desc"]
    """The sorting direction."""

    order: Literal["name", "email", "created_at"]
    """Field to sort by."""

    page: int
    """Current page within paginated list of results."""

    per_page: int
    """The number of results per page. Maximum value is 1000."""

    provenance: Literal["A1S_INTERNAL", "SNOOPY-CASB_OFFICE_365", "SNOOPY-OFFICE_365", "SNOOPY-GOOGLE_DIRECTORY"]

    search: str
    """Search term for filtering records. Behavior may change."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AudioTrackCopyParams"]


class AudioTrackCopyParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    label: Required[str]
    """
    A string to uniquely identify the track amongst other audio track labels for the
    specified video.
    """

    url: str
    """An audio track URL.

    The server must be publicly routable and support `HTTP HEAD` requests and
    `HTTP GET` range requests. The server should respond to `HTTP HEAD` requests
    with a `content-range` header that includes the size of the file.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel
from .skip_configuration import SkipConfiguration

__all__ = ["ContextAwareness"]


class ContextAwareness(BaseModel):
    """
    Scan the context of predefined entries to only return matches surrounded by keywords.
    """

    enabled: bool
    """
    If true, scan the context of predefined entries to only return matches
    surrounded by keywords.
    """

    skip: SkipConfiguration
    """Content types to exclude from context analysis and return all matches."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from ....._models import BaseModel

__all__ = ["ItemEditResponse"]


class ItemEditResponse(BaseModel):
    id: str
    """The unique ID of the allowlist prefix."""

    comment: str
    """An optional comment describing the allowlist prefix."""

    created_on: datetime
    """The creation timestamp of the allowlist prefix."""

    enabled: bool
    """Whether to enable the allowlist prefix into effect. Defaults to false."""

    modified_on: datetime
    """The last modification timestamp of the allowlist prefix."""

    prefix: str
    """The allowlist prefix in CIDR format."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["StreamCreateParams"]


class StreamCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    tus_resumable: Required[Annotated[Literal["1.0.0"], PropertyInfo(alias="Tus-Resumable")]]
    """Specifies the TUS protocol version.

    This value must be included in every upload request. Notes: The only supported
    version of TUS protocol is 1.0.0.
    """

    upload_length: Required[Annotated[int, PropertyInfo(alias="Upload-Length")]]
    """Indicates the size of the entire upload in bytes.

    The value must be a non-negative integer.
    """

    direct_user: bool
    """
    Provisions a URL to let your end users upload videos directly to Cloudflare
    Stream without exposing your API token to clients.
    """

    upload_creator: Annotated[str, PropertyInfo(alias="Upload-Creator")]
    """A user-defined identifier for the media creator."""

    upload_metadata: Annotated[str, PropertyInfo(alias="Upload-Metadata")]
    """Comma-separated key-value pairs following the TUS protocol specification.

    Values are Base-64 encoded. Supported keys: `name`, `requiresignedurls`,
    `allowedorigins`, `thumbnailtimestamppct`, `watermark`, `scheduleddeletion`,
    `maxdurationseconds`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import FileTypes
from ...._utils import PropertyInfo

__all__ = ["DeploymentCreateParams"]


class DeploymentCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    _headers: FileTypes
    """Headers configuration file for the deployment."""

    _redirects: FileTypes
    """Redirects configuration file for the deployment."""

    _routes_json: Annotated[FileTypes, PropertyInfo(alias="_routes.json")]
    """Routes configuration file defining routing rules."""

    _worker_bundle: Annotated[FileTypes, PropertyInfo(alias="_worker.bundle")]
    """Worker bundle file in multipart/form-data format.

    Mutually exclusive with `_worker.js`. Cannot specify both `_worker.js` and
    `_worker.bundle` in the same request. Maximum size: 25 MiB.
    """

    _worker_js: Annotated[FileTypes, PropertyInfo(alias="_worker.js")]
    """Worker JavaScript file.

    Mutually exclusive with `_worker.bundle`. Cannot specify both `_worker.js` and
    `_worker.bundle` in the same request.
    """

    branch: str
    """The branch to build the new deployment from.

    The `HEAD` of the branch will be used. If omitted, the production branch will be
    used by default.
    """

    commit_dirty: Literal["true", "false"]
    """Boolean string indicating if the working directory has uncommitted changes."""

    commit_hash: str
    """Git commit SHA associated with this deployment."""

    commit_message: str
    """Git commit message associated with this deployment."""

    functions_filepath_routing_config_json: Annotated[
        FileTypes, PropertyInfo(alias="functions-filepath-routing-config.json")
    ]
    """Functions routing configuration file."""

    manifest: str
    """JSON string containing a manifest of files to deploy.

    Maps file paths to their content hashes. Required for direct upload deployments.
    Maximum 20,000 entries.
    """

    pages_build_output_dir: str
    """The build output directory path."""

    wrangler_config_hash: str
    """Hash of the Wrangler configuration file used for this deployment."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .custom import (
    CustomResource,
    AsyncCustomResource,
    CustomResourceWithRawResponse,
    AsyncCustomResourceWithRawResponse,
    CustomResourceWithStreamingResponse,
    AsyncCustomResourceWithStreamingResponse,
)
from .domains import (
    DomainsResource,
    AsyncDomainsResource,
    DomainsResourceWithRawResponse,
    AsyncDomainsResourceWithRawResponse,
    DomainsResourceWithStreamingResponse,
    AsyncDomainsResourceWithStreamingResponse,
)
from .managed import (
    ManagedResource,
    AsyncManagedResource,
    ManagedResourceWithRawResponse,
    AsyncManagedResourceWithRawResponse,
    ManagedResourceWithStreamingResponse,
    AsyncManagedResourceWithStreamingResponse,
)

__all__ = [
    "CustomResource",
    "AsyncCustomResource",
    "CustomResourceWithRawResponse",
    "AsyncCustomResourceWithRawResponse",
    "CustomResourceWithStreamingResponse",
    "AsyncCustomResourceWithStreamingResponse",
    "ManagedResource",
    "AsyncManagedResource",
    "ManagedResourceWithRawResponse",
    "AsyncManagedResourceWithRawResponse",
    "ManagedResourceWithStreamingResponse",
    "AsyncManagedResourceWithStreamingResponse",
    "DomainsResource",
    "AsyncDomainsResource",
    "DomainsResourceWithRawResponse",
    "AsyncDomainsResourceWithRawResponse",
    "DomainsResourceWithStreamingResponse",
    "AsyncDomainsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..shared.sort_direction import SortDirection

__all__ = ["RecordListParams", "Comment", "Content", "Name", "Tag"]


class RecordListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    comment: Comment

    content: Content

    direction: SortDirection
    """Direction to order DNS records in."""

    match: Literal["any", "all"]
    """Whether to match all search requirements or at least one (any).

    If set to `all`, acts like a logical AND between filters. If set to `any`, acts
    like a logical OR instead. Note that the interaction between tag filters is
    controlled by the `tag-match` parameter instead.
    """

    name: Name

    order: Literal["type", "name", "content", "ttl", "proxied"]
    """Field to order DNS records by."""

    page: float
    """Page number of paginated results."""

    per_page: float
    """Number of DNS records per page."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    search: str
    """Allows searching in multiple properties of a DNS record simultaneously.

    This parameter is intended for human users, not automation. Its exact behavior
    is intentionally left unspecified and is subject to change in the future. This
    parameter works independently of the `match` setting. For automated searches,
    please use the other available parameters.
    """

    tag: Tag

    tag_match: Literal["any", "all"]
    """Whether to match all tag search requirements or at least one (any).

    If set to `all`, acts like a logical AND between tag filters. If set to `any`,
    acts like a logical OR instead. Note that the regular `match` parameter is still
    used to combine the resulting condition with other filters that aren't related
    to tags.
    """

    type: Literal[
        "A",
        "AAAA",
        "CAA",
        "CERT",
        "CNAME",
        "DNSKEY",
        "DS",
        "HTTPS",
        "LOC",
        "MX",
        "NAPTR",
        "NS",
        "OPENPGPKEY",
        "PTR",
        "SMIMEA",
        "SRV",
        "SSHFP",
        "SVCB",
        "TLSA",
        "TXT",
        "URI",
    ]
    """Record type."""


class Comment(TypedDict, total=False):
    absent: str
    """If this parameter is present, only records _without_ a comment are returned."""

    contains: str
    """Substring of the DNS record comment. Comment filters are case-insensitive."""

    endswith: str
    """Suffix of the DNS record comment. Comment filters are case-insensitive."""

    exact: str
    """Exact value of the DNS record comment. Comment filters are case-insensitive."""

    present: str
    """If this parameter is present, only records _with_ a comment are returned."""

    startswith: str
    """Prefix of the DNS record comment. Comment filters are case-insensitive."""


class Content(TypedDict, total=False):
    contains: str
    """Substring of the DNS record content. Content filters are case-insensitive."""

    endswith: str
    """Suffix of the DNS record content. Content filters are case-insensitive."""

    exact: str
    """Exact value of the DNS record content. Content filters are case-insensitive."""

    startswith: str
    """Prefix of the DNS record content. Content filters are case-insensitive."""


class Name(TypedDict, total=False):
    contains: str
    """Substring of the DNS record name. Name filters are case-insensitive."""

    endswith: str
    """Suffix of the DNS record name. Name filters are case-insensitive."""

    exact: str
    """Exact value of the DNS record name. Name filters are case-insensitive."""

    startswith: str
    """Prefix of the DNS record name. Name filters are case-insensitive."""


class Tag(TypedDict, total=False):
    absent: str
    """Name of a tag which must _not_ be present on the DNS record.

    Tag filters are case-insensitive.
    """

    contains: str
    """A tag and value, of the form `<tag-name>:<tag-value>`.

    The API will only return DNS records that have a tag named `<tag-name>` whose
    value contains `<tag-value>`. Tag filters are case-insensitive.
    """

    endswith: str
    """A tag and value, of the form `<tag-name>:<tag-value>`.

    The API will only return DNS records that have a tag named `<tag-name>` whose
    value ends with `<tag-value>`. Tag filters are case-insensitive.
    """

    exact: str
    """A tag and value, of the form `<tag-name>:<tag-value>`.

    The API will only return DNS records that have a tag named `<tag-name>` whose
    value is `<tag-value>`. Tag filters are case-insensitive.
    """

    present: str
    """Name of a tag which must be present on the DNS record.

    Tag filters are case-insensitive.
    """

    startswith: str
    """A tag and value, of the form `<tag-name>:<tag-value>`.

    The API will only return DNS records that have a tag named `<tag-name>` whose
    value starts with `<tag-value>`. Tag filters are case-insensitive.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .failed_logins import (
    FailedLoginsResource,
    AsyncFailedLoginsResource,
    FailedLoginsResourceWithRawResponse,
    AsyncFailedLoginsResourceWithRawResponse,
    FailedLoginsResourceWithStreamingResponse,
    AsyncFailedLoginsResourceWithStreamingResponse,
)
from .....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from .active_sessions import (
    ActiveSessionsResource,
    AsyncActiveSessionsResource,
    ActiveSessionsResourceWithRawResponse,
    AsyncActiveSessionsResourceWithRawResponse,
    ActiveSessionsResourceWithStreamingResponse,
    AsyncActiveSessionsResourceWithStreamingResponse,
)
from ....._base_client import AsyncPaginator, make_request_options
from .last_seen_identity import (
    LastSeenIdentityResource,
    AsyncLastSeenIdentityResource,
    LastSeenIdentityResourceWithRawResponse,
    AsyncLastSeenIdentityResourceWithRawResponse,
    LastSeenIdentityResourceWithStreamingResponse,
    AsyncLastSeenIdentityResourceWithStreamingResponse,
)
from .....types.zero_trust.access import user_list_params, user_create_params, user_update_params
from .....types.zero_trust.access.user_get_response import UserGetResponse
from .....types.zero_trust.access.user_list_response import UserListResponse
from .....types.zero_trust.access.user_create_response import UserCreateResponse
from .....types.zero_trust.access.user_update_response import UserUpdateResponse

__all__ = ["UsersResource", "AsyncUsersResource"]


class UsersResource(SyncAPIResource):
    @cached_property
    def active_sessions(self) -> ActiveSessionsResource:
        return ActiveSessionsResource(self._client)

    @cached_property
    def last_seen_identity(self) -> LastSeenIdentityResource:
        return LastSeenIdentityResource(self._client)

    @cached_property
    def failed_logins(self) -> FailedLoginsResource:
        return FailedLoginsResource(self._client)

    @cached_property
    def with_raw_response(self) -> UsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return UsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return UsersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        email: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserCreateResponse]:
        """
        Creates a new user.

        Args:
          account_id: Identifier.

          email: The email of the user.

          name: The name of the user.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/access/users", account_id=account_id),
            body=maybe_transform(
                {
                    "email": email,
                    "name": name,
                },
                user_create_params.UserCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserCreateResponse]], ResultWrapper[UserCreateResponse]),
        )

    def update(
        self,
        user_id: str,
        *,
        account_id: str,
        email: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserUpdateResponse]:
        """Updates a specific user's name for an account.

        Requires the user's current email
        as confirmation (email cannot be changed).

        Args:
          account_id: Identifier.

          user_id: UUID.

          email: The email of the user.

          name: The name of the user.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/access/users/{user_id}", account_id=account_id, user_id=user_id),
            body=maybe_transform(
                {
                    "email": email,
                    "name": name,
                },
                user_update_params.UserUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserUpdateResponse]], ResultWrapper[UserUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        email: str | Omit = omit,
        name: str | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[UserListResponse]:
        """
        Gets a list of users for an account.

        Args:
          account_id: Identifier.

          email: The email of the user.

          name: The name of the user.

          page: Page number of results.

          per_page: Number of results per page.

          search: Search for users by other listed query parameters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/access/users", account_id=account_id),
            page=SyncV4PagePaginationArray[UserListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "email": email,
                        "name": name,
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    user_list_params.UserListParams,
                ),
            ),
            model=UserListResponse,
        )

    def delete(
        self,
        user_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Deletes a specific user for an account.

        This will also revoke any active seats
        and tokens for the user.

        Args:
          account_id: Identifier.

          user_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/access/users/{user_id}", account_id=account_id, user_id=user_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        user_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserGetResponse]:
        """
        Gets a specific user for an account.

        Args:
          account_id: Identifier.

          user_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/access/users/{user_id}", account_id=account_id, user_id=user_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserGetResponse]], ResultWrapper[UserGetResponse]),
        )


class AsyncUsersResource(AsyncAPIResource):
    @cached_property
    def active_sessions(self) -> AsyncActiveSessionsResource:
        return AsyncActiveSessionsResource(self._client)

    @cached_property
    def last_seen_identity(self) -> AsyncLastSeenIdentityResource:
        return AsyncLastSeenIdentityResource(self._client)

    @cached_property
    def failed_logins(self) -> AsyncFailedLoginsResource:
        return AsyncFailedLoginsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncUsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncUsersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        email: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserCreateResponse]:
        """
        Creates a new user.

        Args:
          account_id: Identifier.

          email: The email of the user.

          name: The name of the user.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/access/users", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "email": email,
                    "name": name,
                },
                user_create_params.UserCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserCreateResponse]], ResultWrapper[UserCreateResponse]),
        )

    async def update(
        self,
        user_id: str,
        *,
        account_id: str,
        email: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserUpdateResponse]:
        """Updates a specific user's name for an account.

        Requires the user's current email
        as confirmation (email cannot be changed).

        Args:
          account_id: Identifier.

          user_id: UUID.

          email: The email of the user.

          name: The name of the user.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/access/users/{user_id}", account_id=account_id, user_id=user_id),
            body=await async_maybe_transform(
                {
                    "email": email,
                    "name": name,
                },
                user_update_params.UserUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserUpdateResponse]], ResultWrapper[UserUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        email: str | Omit = omit,
        name: str | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[UserListResponse, AsyncV4PagePaginationArray[UserListResponse]]:
        """
        Gets a list of users for an account.

        Args:
          account_id: Identifier.

          email: The email of the user.

          name: The name of the user.

          page: Page number of results.

          per_page: Number of results per page.

          search: Search for users by other listed query parameters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/access/users", account_id=account_id),
            page=AsyncV4PagePaginationArray[UserListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "email": email,
                        "name": name,
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    user_list_params.UserListParams,
                ),
            ),
            model=UserListResponse,
        )

    async def delete(
        self,
        user_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Deletes a specific user for an account.

        This will also revoke any active seats
        and tokens for the user.

        Args:
          account_id: Identifier.

          user_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/access/users/{user_id}", account_id=account_id, user_id=user_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        user_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserGetResponse]:
        """
        Gets a specific user for an account.

        Args:
          account_id: Identifier.

          user_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/access/users/{user_id}", account_id=account_id, user_id=user_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserGetResponse]], ResultWrapper[UserGetResponse]),
        )


class UsersResourceWithRawResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.create = to_raw_response_wrapper(
            users.create,
        )
        self.update = to_raw_response_wrapper(
            users.update,
        )
        self.list = to_raw_response_wrapper(
            users.list,
        )
        self.delete = to_raw_response_wrapper(
            users.delete,
        )
        self.get = to_raw_response_wrapper(
            users.get,
        )

    @cached_property
    def active_sessions(self) -> ActiveSessionsResourceWithRawResponse:
        return ActiveSessionsResourceWithRawResponse(self._users.active_sessions)

    @cached_property
    def last_seen_identity(self) -> LastSeenIdentityResourceWithRawResponse:
        return LastSeenIdentityResourceWithRawResponse(self._users.last_seen_identity)

    @cached_property
    def failed_logins(self) -> FailedLoginsResourceWithRawResponse:
        return FailedLoginsResourceWithRawResponse(self._users.failed_logins)


class AsyncUsersResourceWithRawResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.create = async_to_raw_response_wrapper(
            users.create,
        )
        self.update = async_to_raw_response_wrapper(
            users.update,
        )
        self.list = async_to_raw_response_wrapper(
            users.list,
        )
        self.delete = async_to_raw_response_wrapper(
            users.delete,
        )
        self.get = async_to_raw_response_wrapper(
            users.get,
        )

    @cached_property
    def active_sessions(self) -> AsyncActiveSessionsResourceWithRawResponse:
        return AsyncActiveSessionsResourceWithRawResponse(self._users.active_sessions)

    @cached_property
    def last_seen_identity(self) -> AsyncLastSeenIdentityResourceWithRawResponse:
        return AsyncLastSeenIdentityResourceWithRawResponse(self._users.last_seen_identity)

    @cached_property
    def failed_logins(self) -> AsyncFailedLoginsResourceWithRawResponse:
        return AsyncFailedLoginsResourceWithRawResponse(self._users.failed_logins)


class UsersResourceWithStreamingResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.create = to_streamed_response_wrapper(
            users.create,
        )
        self.update = to_streamed_response_wrapper(
            users.update,
        )
        self.list = to_streamed_response_wrapper(
            users.list,
        )
        self.delete = to_streamed_response_wrapper(
            users.delete,
        )
        self.get = to_streamed_response_wrapper(
            users.get,
        )

    @cached_property
    def active_sessions(self) -> ActiveSessionsResourceWithStreamingResponse:
        return ActiveSessionsResourceWithStreamingResponse(self._users.active_sessions)

    @cached_property
    def last_seen_identity(self) -> LastSeenIdentityResourceWithStreamingResponse:
        return LastSeenIdentityResourceWithStreamingResponse(self._users.last_seen_identity)

    @cached_property
    def failed_logins(self) -> FailedLoginsResourceWithStreamingResponse:
        return FailedLoginsResourceWithStreamingResponse(self._users.failed_logins)


class AsyncUsersResourceWithStreamingResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.create = async_to_streamed_response_wrapper(
            users.create,
        )
        self.update = async_to_streamed_response_wrapper(
            users.update,
        )
        self.list = async_to_streamed_response_wrapper(
            users.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            users.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            users.get,
        )

    @cached_property
    def active_sessions(self) -> AsyncActiveSessionsResourceWithStreamingResponse:
        return AsyncActiveSessionsResourceWithStreamingResponse(self._users.active_sessions)

    @cached_property
    def last_seen_identity(self) -> AsyncLastSeenIdentityResourceWithStreamingResponse:
        return AsyncLastSeenIdentityResourceWithStreamingResponse(self._users.last_seen_identity)

    @cached_property
    def failed_logins(self) -> AsyncFailedLoginsResourceWithStreamingResponse:
        return AsyncFailedLoginsResourceWithStreamingResponse(self._users.failed_logins)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["BrowserCacheTTL"]


class BrowserCacheTTL(BaseModel):
    id: Optional[Literal["browser_cache_ttl"]] = None
    """Control how long resources cached by client browsers remain valid."""

    value: Optional[int] = None
    """
    The number of seconds to cache resources for. Setting this to 0 enables "Respect
    Existing Headers".
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.r2_data_catalog import credential_create_params

__all__ = ["CredentialsResource", "AsyncCredentialsResource"]


class CredentialsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CredentialsResourceWithStreamingResponse(self)

    def create(
        self,
        bucket_name: str,
        *,
        account_id: str,
        token: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Store authentication credentials for a catalog.

        These credentials are used to
        authenticate with R2 storage when performing catalog operations.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          token: Provides the Cloudflare API token for accessing R2.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}/credential",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            body=maybe_transform({"token": token}, credential_create_params.CredentialCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class AsyncCredentialsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCredentialsResourceWithStreamingResponse(self)

    async def create(
        self,
        bucket_name: str,
        *,
        account_id: str,
        token: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Store authentication credentials for a catalog.

        These credentials are used to
        authenticate with R2 storage when performing catalog operations.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          token: Provides the Cloudflare API token for accessing R2.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}/credential",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            body=await async_maybe_transform({"token": token}, credential_create_params.CredentialCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class CredentialsResourceWithRawResponse:
    def __init__(self, credentials: CredentialsResource) -> None:
        self._credentials = credentials

        self.create = to_raw_response_wrapper(
            credentials.create,
        )


class AsyncCredentialsResourceWithRawResponse:
    def __init__(self, credentials: AsyncCredentialsResource) -> None:
        self._credentials = credentials

        self.create = async_to_raw_response_wrapper(
            credentials.create,
        )


class CredentialsResourceWithStreamingResponse:
    def __init__(self, credentials: CredentialsResource) -> None:
        self._credentials = credentials

        self.create = to_streamed_response_wrapper(
            credentials.create,
        )


class AsyncCredentialsResourceWithStreamingResponse:
    def __init__(self, credentials: AsyncCredentialsResource) -> None:
        self._credentials = credentials

        self.create = async_to_streamed_response_wrapper(
            credentials.create,
        )
